
################################################################################################################
# Creating an illustrative data set (N = 200 scores, negatively skewed)

Illustrative.Data <- round(100 - 2.5 * qchisq(seq(.003, .997, length = 200), df = 10))

# Random series of correct (1) vs. incorrect (0) responses to the item to be dropped in method # 4
set.seed(1)
N <- length(Illustrative.Data)
Illustrative.X2 <- rep(0, N)
for (i in 1:N) 
   Illustrative.X2[i] <- 1 * runif(1) < (Illustrative.Data[i] / 285)


################################################################################################################
Adjust <- function(X = Illustrative.Data, Method, Round = T,
                   C = 5, 
                   H1 = 90, 
                   X2 = Illustrative.X2, PV = 5, 
                   PP = 25, 
                   Min = 0, Max = 100, S = 10,
                   A = .5,
                   M = 82, SD = 8,
                   G = 20)
{
   # X = original scores (length = N); by default, a negatively skewed distribution is used (see above)
   # Method = adjustment method (see below)
   # Round = whether or not to ensure that adjusted scores are whole numbers (default = T)

   # Methods + required parameters:
   #   1:  no adjustment
   #   2:  adding a constant; C = points to add
   # 3.1:  percent of highest score
   # 3.2:  percent of highest score; H1 = self-created highest score
   #   4:  remove poor item; X2 = series of 0s (incorrect) + 1s (correct) corresponding to each original score; 
   #       PV = point value of item to be removed
   #   5:  give percent of points back; PP = percent of points to return
   #   6:  stretching function; Min = minimum value for arc; Max = maximum value for arc; S = stretching value
   #   7:  root of score; A = root
   #   8:  setting M + SD; M = desired mean; SD = desired SD
   #   9:  percentile transformation; M = desired mean; SD = desired SD
   #  10:  control for guessing; G = percent of points expected by guessing at random
   
   if (Method == 1) Y <- X
   if (Method == 2) Y <- X + C
   if (Method == 3.1) Y <- 100 * X / max(X)
   if (Method == 3.2) Y <- 100 * X / H1
   if (Method == 4) Y <- 100 * (X - X2 * PV) / (100 - PV)
   if (Method == 5) Y <- X + (100 - X) * (PP / 100)
   if (Method == 6) Y <- X + (((X - Min) * (Max - X)) / ((Max - Min) / 2) ^ 2) * S
   if (Method == 7) Y <- 100 ^ (1 - A) * X ^ A
   if (Method == 8) Y <- (X - mean(X)) / sd(X) * SD + M
   if (Method == 9) Y <- qnorm((rank(X) - .5) / length(X)) * SD + M
   if (Method == 10) Y <- 100 * (X - G) / (100 - G)

   if (Round) Y <- round(Y)   

#   dev.new(height = 4, width = 12)
   dev.new(height = 3, width = 9)
   par(mfrow = c(1, 3), omi = c(.1, .1, .1, .1), mai = c(.7, .7, .7, .2), cex = .8)

   # plot 1 = adjustment function
   Mins <- min(X, Y)
   Maxs <- max(X, Y)
   plot(x = X, y = Y, xlim = c(Mins, Maxs), ylim = c(Mins, Maxs), type = "p", pch = 19, 
         xlab = "Scores", ylab = "Grades", main = "Adjustment Function")
   lines(x = c(Mins, Maxs), y = c(Mins, Maxs), lty = 3)
   lines(x = c(Mins, 100), y = c(100, 100), lty = 3)
   
   # plot 2 = grade distributions
   Min.X <- min(min(density(X)$x), min(density(Y)$x))
   Max.X <- max(max(density(X)$x), max(density(Y)$x))
   Max.Y <- max(max(density(X)$y), max(density(Y)$y))
   plot(density(Y), xlim = c(Min.X, Max.X), ylim = c(0, Max.Y), lwd = 2,
         xlab = "Scores", ylab = "Density", main = "Grade Distribution")
   lines(density(X), lty = 3)

   # plot 3 = grade distribution
   plot(x = c(0, 100), y = c(10, 100), xaxt = "n", yaxt = "n", type = "n", 
         xlab = "", ylab = "", main = "Grade Distribution")
   text(x = 25, y = 90, paste("Min =", summary(Y)[1]))
   text(x = 25, y = 80, paste("Q1 =", summary(Y)[2]))
   text(x = 25, y = 70, paste("Mdn =", summary(Y)[3]))
   text(x = 25, y = 60, paste("Q3 =", summary(Y)[5]))
   text(x = 25, y = 50, paste("Max =", summary(Y)[6]))

   text(x = 25, y = 30, paste("M =", round(mean(Y), 2)))
   text(x = 25, y = 20, paste("SD =", round(sd(Y), 2)))

   text(x = 75, y = 90, "Percent in Each")
   text(x = 75, y = 80, "Grade Range")
   text(x = 65, y = 60, paste("A: ", round(100 * mean(Y >= 90))), pos = 4)
   text(x = 65, y = 50, paste("B: ", round(100 * mean((Y >= 80) & (Y < 90)))), pos = 4)
   text(x = 65, y = 40, paste("C: ", round(100 * mean((Y >= 70) & (Y < 80)))), pos = 4)
   text(x = 65, y = 30, paste("D: ", round(100 * mean((Y >= 60) & (Y < 70)))), pos = 4)
   text(x = 65, y = 20, paste("F: ", round(100 * mean(Y < 60))), pos = 4)

}



################################################################################################################
# Calculating point of maximum increase for "root of score" adjustment method

Max.Inc <- function(a)
{
   x <- seq(from = 0, to = 100, by = .1)
   y <- 100 ^ (1 - a) * x ^ a
   diff <- y - x
   plot(x, y, ylim = c(0,100), type = "l", lty = 1, lwd = 2, xlab = "Score", ylab = "Grade")
   lines(x, diff, lty = 1)
   lines(x = c(0, 100), y = c(0, 100), lty = 2, lwd = 2)
   lines(x = c(0, 100), y = c(0, 0), lty = 2)
   max.x <- (1 / (a * 100 ^ (1 - a))) ^ (1 / (a - 1))
   abline(v = max.x)
   return(max.x)
}

################################################################################################################
# Reliability simulation

Reliability <- function(N = 200, Items = 25, Reps = 1000)
{
   set.seed(1)
   r.easy <- r.mod <- r.diff <- rep(0, Reps)
   score.easy <- score.mod <- score.diff <- matrix(0, Reps, N)
   ability <- rnorm(N)
   items.easy <- runif(Items, -.5, .0)
   items.mod <- runif(Items, -1.5, -1)
   items.diff <- runif(Items, -2.5, -2)
   corr.matrix <- matrix(0, 3, 3)
   Row.Labels <- Col.Labels <- c("corr(E)", "corr(M)", "corr(D)")
   dimnames(corr.matrix) <- list(Row.Labels, Col.Labels)
   for (i in 1:Reps)
   {
      x.easy <- x.mod <- x.diff <- matrix(0, N, Items)
      for (j in 1:Items)
         for (k in 1:N)
         {
            resp <- rnorm(1, ability[k], 2)
            x.easy[k,j] <- 1 * resp > items.easy[j]
            x.mod[k,j] <- 1 * resp > items.mod[j]
            x.diff[k,j] <- 1 * resp > items.diff[j]
         }
      score.easy[i,] <- round(100 * apply(x.easy, 1, sum) / Items)
      score.mod[i,] <- round(100 * apply(x.mod, 1, sum) / Items)
      score.diff[i,] <- round(100 * apply(x.diff, 1, sum) / Items)
      r.easy[i] <- Alpha(x.easy)
      r.mod[i] <- Alpha(x.mod)
      r.diff[i] <- Alpha(x.diff)
      corr.matrix <- corr.matrix + (1/Reps) * cor(cbind(score.easy[i,], score.mod[i,], score.diff[i,]), method = "spearman")
   }
   cat("\nN =",N,"\n")
   cat("Items =",Items,"\n")
   cat("Replications =",Reps,"\n\n")
   Results <- matrix(0, 3, 2)
   Row.Labels <- c("Easy", "Moderate", "Difficult")
   Col.Labels <- c("Test Score", "Alpha")
   dimnames(Results) <- list(Row.Labels, Col.Labels)
   Results[1,1] <- mean(score.easy)
   Results[2,1] <- mean(score.mod)
   Results[3,1] <- mean(score.diff)
   Results[1,2] <- mean(r.easy)
   Results[2,2] <- mean(r.mod)
   Results[3,2] <- mean(r.diff)
   Results <- cbind(Results, corr.matrix)
   print(round(Results,4))
   xs <- c(apply(score.easy, 1, mean),apply(score.mod, 1, mean),apply(score.diff, 1, mean))
   ys <- c(r.easy,r.mod,r.diff)
   plot(xs, ys, xlab = "Mean Test Score", ylab = "Internal Consistency")
   cat("\nCorrelation between test score and alpha =",round(cor(xs,ys),4),"\n")
}

################################################################################################################
# Cronbach's alpha

Alpha <- function(x)
{
   N <- dim(x)[1]
   k <- dim(x)[2]
   item.var <- sum(apply(x, 2, var))
   score <- apply(x, 1, sum)
   alpha <- (k / (k - 1)) * (1 - (item.var / var(score)))
   return(alpha)
}
